import Socket from "../../../../api/Socket";
import { ChannelInfo } from '../../../channel/channelDefinitions';
import { AccessConfig } from './accessConfigs';
import { Input } from './inputConfig';
import { BackErrorCollectionLimitConfig } from './backErrorCollectionLimitConfig';
export declare type ChSubAccessFunction = (socket: Socket, info: ChannelInfo) => Promise<boolean> | boolean;
export interface ChannelConfig extends AccessConfig<ChSubAccessFunction>, BackErrorCollectionLimitConfig {
    /**
     * This property defines the member input.
     * This option allows you to define your custom composite members from different keys.
     * The client sends the member when it builds the connection to a Channel.
     * In a Channel, the member value is available in almost every function.
     * By default, any string value is allowed.
     * But you can use a model to define the member or allow any input with the 'any' literal.
     * Notice, the member will only be validated with the model but not transformed to
     * avoid member types' confusion and improve performance.
     * So it is not possible to add functions to member value.
     * @default value of type string
     * @example
     * @ObjectModel()
     * class CompositeChatMember {
     *  userId = Model({type: 'string'});
     *  chatId = Model({type: 'string'});
     * }
     * memberInput: CompositeChatMember
     * //Client can send  ->
     * {userId: 'Luca', chatId: 'sf0ij23r'}
     */
    memberInput?: Input;
    /**
     * Defines the delay to unregister the StaticChannel or a member of the Channel
     * internally when no one uses it anymore.
     * When a client starts to use it again, the delay timeout will be cancelled.
     * @default 120000ms
     */
    unregisterDelay?: number;
    /**
     * The maximal amount of members where a socket can connect to within this Channel.
     * Notice this option is only used in Channel Families.
     * @default 20
     */
    maxSocketMembers?: number;
    /**
     * Defines the batch option for sending data to the sockets of the channel.
     * It helps to package packets together and can improve performance but also adds a bit of latency.
     * Notice that batching is only used for publishing, and batching will not work when using the publisher option.
     * A number specifies the maximum amount of milliseconds a package waits in the buffer
     * to be bundled with other packages before sending it.
     * Notice that the package may be sent earlier than the specified milliseconds.
     * When setting the batch option to null, packages will not be bundled and send directly.
     * In cases of high traffic, this could improve the performance.
     * @default 100
     */
    batch?: null | number;
}
