import { Server } from "ziron-worker";
import Socket from "../../../../api/Socket";
import CodeError from "../../../error/codeError";
import { RawSocket } from '../../../definitions/rawSocket';
export declare type CoreServerFunction = (server: Server<RawSocket>) => Promise<void> | void;
export declare type CoreServerErrorFunction = (error: Error, server: Server<RawSocket>) => Promise<void> | void;
export declare type InitFunction = () => Promise<void> | void;
export declare type BeforeComponentsInitFunction = () => Promise<void> | void;
export declare type AfterComponentsInitFunction = () => Promise<void> | void;
export declare type StartedFunction = () => Promise<void> | void;
export declare type CleanupFunction = () => Promise<void> | void;
export declare type ErrorFunction = (error: Error) => Promise<void> | void;
export declare type CodeErrorFunction = (codeError: CodeError) => Promise<void> | void;
export declare type ServerMessageFunction = (data: any) => Promise<void> | void;
export declare type SocketInitFunction = (socket: Socket) => Promise<void> | void;
export declare type SocketConnectionFunction = (socket: Socket) => Promise<void> | void;
export declare type SocketDisconnectionFunction = (socket: Socket, code: any, data: any) => Promise<void> | void;
export declare type SocketAuthenticationFunction = (socket: Socket) => Promise<void> | void;
export declare type SocketDeauthenticationFunction = (socket: Socket) => Promise<void> | void;
export declare type CoreSocketErrorFunction = (error: object, socket: Socket) => Promise<void> | void;
export declare type SocketBadAuthTokenFunction = (socket: Socket, error: Error, signedAuthToken: string) => Promise<void> | void;
export declare type Event<T> = T | T[];
export interface Events {
    /**
     * Core server event that will be invoked in the server start to initialize something with the underlying ziron server.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (server) => {}
     */
    coreServer?: Event<CoreServerFunction>;
    /**
     * This gets triggered when an error occurs on a core server.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (error,server) => {}
     */
    coreServerError?: Event<CoreServerErrorFunction>;
    /**
     * An event that can be used to do extra things in the startup of the server.
     * The server startup process will wait for the promise of this event to be resolved.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example async () => {}
     */
    init?: Event<InitFunction>;
    /**
     * An event that will be processed before all components will be initialized.
     * It is useful to prepare things for the initialize of the components.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example async () => {}
     */
    beforeComponentsInit?: Event<BeforeComponentsInitFunction>;
    /**
     * An event that will be processed after all components have initialized.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example async (leaderWorker) => {}
     */
    afterComponentsInit?: Event<AfterComponentsInitFunction>;
    /**
     * An event that gets invoked when a server is started.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example () => {}
     */
    started?: Event<StartedFunction>;
    /**
     * An event that gets invoked before the server exists.
     * It can be used to clean up resources.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example () => {}
     */
    cleanup?: Event<CleanupFunction>;
    /**
     * An event that gets invoked when an unknown error is thrown on the server while:
     * processing a request, run a background task or processing another event.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (error) => {}
     */
    error?: Event<ErrorFunction>;
    /**
     * An event that gets invoked when a CodeError is created.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (codeError) => {}
     */
    codeError?: Event<CodeErrorFunction>;
    /**
     * An event that gets invoked when the server receives a server message
     * that was sent from the bag.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (data) => {}
     */
    serverMessage?: Event<ServerMessageFunction>;
    /**
     * An event that can be used to do extra things in the creation process of a socket.
     * Notice that a returned promise will not be awaited.
     * This event is only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketInit?: Event<SocketInitFunction>;
    /**
     * An event that gets invoked when a new socket is connected to the server.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketConnection?: Event<SocketConnectionFunction>;
    /**
     * An event that gets invoked when a socket gets disconnected.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket,code,data) => {}
     */
    socketDisconnection?: Event<SocketDisconnectionFunction>;
    /**
     * An event that gets invoked when a socket gets authenticated or the auth token is changed.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketAuthentication?: Event<SocketAuthenticationFunction>;
    /**
     * An event that gets invoked when a socket gets deauthenticated.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketDeauthentication?: Event<SocketDeauthenticationFunction>;
    /**
     * This gets triggered when an error occurs on a core socket.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (error,socket) => {}
     */
    coreSocketError?: Event<CoreSocketErrorFunction>;
    /**
     * Emitted when a client tries to authenticate itself with an invalid (or expired) token.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket,error,signedAuthToken) => {}
     */
    socketBadAuthToken?: Event<SocketBadAuthTokenFunction>;
}
export interface PreparedEvents extends Events {
    coreServer: CoreServerFunction;
    coreServerError: CoreServerErrorFunction;
    init: InitFunction;
    beforeComponentsInit: BeforeComponentsInitFunction;
    afterComponentsInit: AfterComponentsInitFunction;
    started: StartedFunction;
    cleanup: CleanupFunction;
    error: ErrorFunction;
    codeError: CodeErrorFunction;
    serverMessage: ServerMessageFunction;
    socketInit: SocketInitFunction;
    socketConnection: SocketConnectionFunction;
    socketDisconnection: SocketDisconnectionFunction;
    socketAuthentication: SocketAuthenticationFunction;
    socketDeauthentication: SocketDeauthenticationFunction;
    coreSocketError: CoreSocketErrorFunction;
    socketBadAuthToken: SocketBadAuthTokenFunction;
}
